/*********************************************************************
 * IBM Confidential
 *
 * OCO Source Materials
 * 
 * --SA4.1_PID--
 * Copyright IBM Corp. 2005, 2006, 2007, 2008
 * The source code for this program is not published or otherwise 
 * divested of its trade secrets, irrespective of what
 * been deposited with the U.S. Copyright Office. 
 *-----------------------------------------------				
 * Author: Surren Partabh
 * Created: 06/01/2005
 * Description:
 *   Script to create the administration package bodies
 *-----------------------------------------------				
 * MODIFICATION HISTORY
 ********************************************************************/

CREATE OR REPLACE PACKAGE BODY
 ADM_Date IS

   g_StartOfWeek INTEGER := 0;

   -- --------------------------------------------------
   -- Simple years support ( 12 months )
   -- This will be correct in most cases, but perhaps
   -- this should add years based on ISO Year (TODO?)...
   -- INTERNAL
   -- --------------------------------------------------
   FUNCTION add_years( pDate IN DATE, n IN INTEGER ) RETURN DATE IS
   BEGIN
      return add_months( pDate, n * 12 );
   END add_years;

   -- -----------------------------------------------------------------------
   -- get the week offset. ( computed from start of week )
   -- -----------------------------------------------------------------------
   FUNCTION WeekOffset RETURN INTEGER IS
   BEGIN
      SELECT DECODE (TO_CHAR(TRUNC(SYSDATE,'D'),'DY','NLS_DATE_LANGUAGE=AMERICAN'),'MON',1,7)
           into g_StartOfWeek FROM dual;
      RETURN MOD( 8 - g_StartOfWeek, 7 );
   END WeekOffset;


   -- -----------------------------------------------------------------------
   -- trunc_to_last_period - Truncate tstamp down to the previous period.
   -- -----------------------------------------------------------------------
   FUNCTION trunc_to_last_period(tstamp DATE, period CHAR) RETURN DATE IS
      truncMask VARCHAR2(5);
      truncTime DATE;
      today DATE := trunc(tstamp, 'DD');
      secondsPerDay CONSTANT INTEGER := 24 * 3600;
      offset NUMBER := 0;

   BEGIN
	 IF (period = 'D') THEN
	    truncMask := 'DDD';
	 ELSIF (period = 'W') THEN
	    offset    := WeekOffset();
	    truncMask := 'IW';
	 ELSIF (period = 'M') THEN
	    truncMask := 'MON';
         ELSIF (period = 'Y') THEN
	    truncMask := 'IY';
	    offset    := WeekOffset();
	 ELSIF (period = 'H') THEN
	    truncMask := 'HH24';
	 ELSIF (period = 'P') THEN
	    truncTime := today +
	       floor((tstamp - today) * 60 * 60 * 24 / 1800) * 1800 / secondsPerDay;
	 ELSIF (period = 'Q') THEN
	    truncTime := today +
	       floor((tstamp - today) * 60 * 60 * 24 / 900) * 900 / secondsPerDay;
	 ELSIF (period = 'T') THEN
	    truncTime := today +
	       floor((tstamp - today) * 60 * 60 * 24 / 600) * 600 / secondsPerDay;
         ELSIF (period = 'F') THEN
	    truncTime := today +
	       floor((tstamp - today) * 60 * 60 * 24 / 300) * 300 / secondsPerDay;
	 ELSIF (period = 'B') THEN
	    truncTime := today +
	       floor((tstamp - today) * 60 * 60 * 24 / 120) * 120 / secondsPerDay;
        ELSIF (period = 'O') THEN
	    truncTime := today +
	       floor((tstamp - today) * 60 * 60 * 24 / 60) * 60 / secondsPerDay;
	 ELSE
	    RETURN null;
	 END IF;

	 IF (truncTime IS NOT NULL) THEN
	    RETURN truncTime;
	 ELSIF (truncMask='IW' OR truncMask='IY') THEN
	    RETURN trunc(tstamp+offset,truncMask)-offset;
	 ELSE
	    RETURN trunc(tstamp, truncMask);
	 END IF;

   END trunc_to_last_period;

   -- -----------------------------------------------------------------------
   -- add_period - Adds N periods to tstamp
   -- -----------------------------------------------------------------------
   FUNCTION add_periods(tstamp DATE, period CHAR, n INTEGER DEFAULT 1)
			RETURN DATE IS
      retVal DATE;
   BEGIN

      IF (period = 'D') THEN
	 retVal := tstamp + n;
      ELSIF (period = 'W') THEN
	 retVal := tstamp + (n * 7);
      ELSIF (period = 'M') THEN
	 retVal := add_months(tstamp, n);
      ELSIF (period = 'Y') THEN
         retVal := add_years(tstamp,n);
      ELSIF (period = 'H') THEN
	 retVal := tstamp + n/24;
      ELSIF (period = 'P') THEN
	 retVal := tstamp + n/48;
      ELSIF (period = 'Q') THEN
	 retVal := tstamp + n/96;
      ELSIF (period = 'T') THEN
	 retVal := tstamp + n/144;
      ELSIF (period = 'F') THEN
	 retVal := tstamp + n/288;
      ELSIF (period = 'B') THEN
	 retVal := tstamp + n/720;
      ELSIF (period = 'O') THEN
	 retVal := tstamp + n/1440;
      ELSE
	 retVal := NULL;
      END IF;

      RETURN retVal;

   END add_periods;


   -- -----------------------------------------------------------------------
   -- subtract_period - subtract N periods to tstamp
   -- -----------------------------------------------------------------------
   FUNCTION subtract_periods(tstamp DATE, period CHAR, n INTEGER DEFAULT 1)
			     RETURN DATE IS
      retVal DATE;
   BEGIN

      retVal := add_periods(tstamp, period, -1 * n);

      RETURN retVal;

   END subtract_periods;

   -- -----------------------------------------------------------------------
   -- period_to_name - Translates a period CHAR to a full name
   -- -----------------------------------------------------------------------
   FUNCTION period_to_name(period CHAR) RETURN VARCHAR2 IS
      retVal VARCHAR2(10);
   BEGIN

      IF (period = 'D') THEN
	 retVal := 'daily';
      ELSIF (period = 'W') THEN
	 retVal := 'weekly';
      ELSIF (period = 'M') THEN
	 retVal := 'monthly';
      ELSIF (period = 'Y') THEN
         retVal := 'yearly';
      ELSIF (period = 'H') THEN
	 retVal := 'hourly';
      ELSIF (period = 'P') THEN
	 retVal := 'HalfHour';
      ELSIF (period = 'Q') THEN
	 retVal := 'QuarterHour';
      ELSIF (period = 'T') THEN
	 retVal := 'TenMinutes';
      ELSIF (period = 'F') THEN
	 retVal := 'FiveMinutes';
      ELSIF (period = 'B') THEN
	 retVal := 'TwoMinutes';
      ELSIF (period = 'O') THEN
	 retVal := 'OneMinute';
      ELSE
	 retVal := NULL;
      END IF;

      RETURN retVal;

   END period_to_name;


   -- -----------------------------------------------------------------------
   -- calc_end_date - Calculate an end date N periods from a start date.
   -- Because BETWEEN is inclusive we need to back off one second to exclude
   -- everything from the next period.
   -- -----------------------------------------------------------------------
   FUNCTION calc_end_date(tstamp DATE, period CHAR, n INTEGER DEFAULT 1)
			  RETURN DATE IS
      retVal DATE;
   BEGIN

      retVal := add_periods(tstamp, period, n);
      retVal := retVal - 1/(60*60*24);

      RETURN retVal;

   END calc_end_date;

END ADM_Date;
/
SHOW ERROR

