/*********************************************************************
 * IBM Confidential
 *
 * OCO Source Materials
 * 
 * --SA4.1_PID--
 * Copyright IBM Corp. 2005, 2006, 2007, 2008
 * The source code for this program is not published or otherwise 
 * divested of its trade secrets, irrespective of what
 * been deposited with the U.S. Copyright Office. 
 *-----------------------------------------------				
 * Author: Surren Partabh
 * Created: 06/01/2005
 * Description:
 *-----------------------------------------------				
 * MODIFICATION HISTORY
* DATE		MODIFIED BY	MODIFICATION
 ********************************************************************/

/* PROC_ANALYZE_STALE : This procedure analyzes stale database objects based
 * on the IO activity on the object. All tables must be in a MONITORed state.
 * Statistics are gathered in various ways including on a global level, a
 * partitioned level and on all tables that have stale or non-existing statistics
 * associated with them.
 * DBMS_STATS is used to gather statistics and CASCADE should always be TRUE
 * to ensure that statistics are gathered on all associated indexes.
 * Statistics are gathered only on the table / partition with activity on it
 */

PROMPT Creating Procedure 'PROC_ANALYZE_STALE'
CREATE OR REPLACE PROCEDURE proc_analyze_stale (
   l_schema_owner   IN   VARCHAR2
)
AS
/* CURSOR l_stale_tab_cur : The l_stale_tab_cur cursor stores
 * details of all monitored tables with IO activity on them from the
 * user_tab_modifications table. This table is populated when
 * monitoring is turned on on a table.
 */
   CURSOR l_stale_tab_cur (
       b_table_owner IN all_tab_modifications.table_owner%TYPE
   ) IS
      SELECT DISTINCT
          table_name,
          partition_name
      FROM
          all_tab_modifications
      WHERE
          table_owner = b_table_owner
      ORDER BY
          table_name;


/* CURSOR l_list_stale_cur : The l_list_stale_cur cursor stores
 * details of all the tables which have stale statistics
 */
   CURSOR l_list_stale_cur (
       b_table_owner IN all_tables.owner%TYPE
   ) IS
       SELECT
           table_name
       FROM
           all_tables
       WHERE
           owner = b_table_owner
       AND
           partitioned = 'YES'
       AND
           (last_analyzed <= TO_DATE (SYSDATE-6/24) OR (last_analyzed IS NULL))
       ORDER BY 
           TABLE_NAME;


/* CURSOR l_get_stats_cur : The l_get_stats_cur cursor stores
 * details of the total number of rows and blocks used by a
 * tables partitons with the aim of using them to popular the
 * tables global statistics.
 */
   CURSOR l_get_stats_cur (
       b_table_name IN all_tab_partitions.table_name%TYPE
   ) IS
       SELECT
           table_name,
           SUM(num_rows) sum_rows,
           SUM(blocks) sum_blocks,
           AVG(avg_row_len) avg_rlen
       FROM
           all_tab_partitions
       WHERE
           table_name= b_table_name
       AND
           num_rows <> 0
       GROUP BY table_name;

   l_owner_name            VARCHAR2 (30);
   v_job_date              DATE:=SYSDATE;
   l_analyze_str          VARCHAR2 (255);
   l_part_prefix            VARCHAR2 (5)  := 'PART_';
   l_part_name            VARCHAR2 (15);
   l_proc_name CONSTANT           VARCHAR2(30) := 'PROC_ANALYZE_STALE';
   l_system_global_sample_size    INTEGER := 5;
   l_system_partition_sample_size INTEGER := 20;
   l_global_sample_size           INTEGER;
   l_partition_sample_size        INTEGER;
   l_degree                       INTEGER := 4;
   b_block_sampling               BOOLEAN := TRUE;
   l_block_sampling_flag          VARCHAR2(10);
   b_system_block_sampling        BOOLEAN := TRUE;
   l_system_block_sampling_flag   INTEGER := 1;
   b_analyze                      BOOLEAN := TRUE;
   l_period                       VARCHAR2(10);

   FUNCTION get_tunable(param_name IN VARCHAR2, param_default IN INTEGER)
        RETURN INTEGER
   IS   
        parameter_val INTEGER;
   BEGIN
      SELECT TRUNC( TO_NUMBER( value ) ) INTO parameter_val
         FROM adm_system_variables
         WHERE UPPER(name) = UPPER( param_name );

      RETURN NVL(parameter_val, param_default);
   EXCEPTION
     WHEN OTHERS THEN
       RETURN param_default;
   END;


BEGIN
    log_pkg.log_info_table(l_proc_name, NULL, NULL, NULL, sysdate, 'Started');

    l_system_global_sample_size := get_tunable('GlobalAnalyzePct',l_system_global_sample_size);
    l_system_partition_sample_size := get_tunable('PartitionAnalyzePct',l_system_partition_sample_size);
    l_degree := get_tunable('AnalyzeDegree',l_degree);

    l_system_block_sampling_flag := get_tunable('BlockSampling',l_system_block_sampling_flag);
    IF (l_system_block_sampling_flag < 1) THEN
	b_system_block_sampling := FALSE;
    ELSE
	b_system_block_sampling := TRUE;
    END IF;

    l_owner_name    := RTRIM (UPPER (l_schema_owner));

    /* Flush Monitoring Stats to _TAB_MODIFICATIONS tables */
    DBMS_STATS.FLUSH_DATABASE_MONITORING_INFO;

    /* Gather Global or Partitioned Statistics on Monitored Tables with activity on the whole
     * table or on a particular partition. For loop loops through all tables which appear in
     * the user_tab_modifications table and gather statistics on them.
     */

    FOR l_stale_tab_rec IN l_stale_tab_cur (l_owner_name)
    LOOP

       IF (l_stale_tab_rec.table_name != 'EMPTY') AND (l_stale_tab_rec.partition_name IS NOT NULL )
       THEN
           /* Gather Statistics on Table Partitions that have activity on them */
	   b_analyze := TRUE;
	   BEGIN
	      SELECT partition_sample_size, block_sample, period INTO
                 l_partition_sample_size, l_block_sampling_flag, l_period
	         FROM part_tables WHERE table_name = UPPER(l_stale_tab_rec.table_name) AND rownum < 2;
	      IF l_block_sampling_flag = 'Y' THEN 
       	         b_block_sampling := TRUE;
	      ELSE 
		 b_block_sampling := FALSE;
	      END IF;
	      IF l_period = 'Q' THEN 
	         b_analyze := FALSE;
	      END IF;
	   EXCEPTION
	      WHEN NO_DATA_FOUND THEN
	          -- use global defaults
		  l_partition_sample_size := l_system_partition_sample_size;
                  b_block_sampling := b_system_block_sampling;
	   END;

	   IF b_analyze = TRUE THEN
              DBMS_Output.Put_Line('Gathering partitioned stats:'||l_stale_tab_rec.table_name||':'||l_stale_tab_rec.partition_name);
              log_pkg.log_info_table(l_proc_name, l_stale_tab_rec.table_name||':'||l_stale_tab_rec.partition_name, NULL, NULL, sysdate, 'Gathering partitioned stats : sample size '||l_partition_sample_size||', degree '||l_degree||'...');

              BEGIN
                  DBMS_STATS.gather_table_stats
                   (ownname                     => l_owner_name,
                    tabname                     => l_stale_tab_rec.table_name,
                    partname                    => l_stale_tab_rec.partition_name,
                    estimate_percent            => l_partition_sample_size,
                    block_sample                => b_block_sampling,
                    method_opt                  => 'FOR ALL COLUMNS SIZE 1',
                    degree                      => l_degree,
                    granularity                 => 'PARTITION',
                    cascade                     => TRUE
                   );
                  log_pkg.log_info_table(l_proc_name, l_stale_tab_rec.table_name||':'||l_stale_tab_rec.partition_name, NULL, NULL, sysdate, 'Gathered partitioned stats');

              EXCEPTION WHEN OTHERS THEN
               log_pkg.error('PROC_ANALYZE_STALE.PARTITIONED_STATS,'||l_owner_name, SQLCODE, RTRIM (SQLERRM), l_stale_tab_rec.table_name||':'||l_stale_tab_rec.partition_name);
              END;
	   END IF;

       ELSIF (l_stale_tab_rec.table_name != 'EMPTY') AND (l_stale_tab_rec.partition_name IS NULL )
       THEN
	   b_analyze := TRUE;
	   BEGIN
	      SELECT global_sample_size, block_sample, period INTO
	          l_global_sample_size, l_block_sampling_flag, l_period
		  FROM part_tables WHERE table_name = UPPER(l_stale_tab_rec.table_name) AND rownum < 2;
	      IF l_block_sampling_flag = 'Y' THEN 
	          b_block_sampling := TRUE;
	      ELSE 
		  b_block_sampling := FALSE;
	      END IF;
	      IF l_period = 'Q' THEN 
		  b_analyze := FALSE;
	      END IF;
	   EXCEPTION
	      WHEN NO_DATA_FOUND THEN
		-- use global defaults
	        l_global_sample_size := l_system_global_sample_size;
                b_block_sampling := b_system_block_sampling;
	   END;          /* Gather Global Statistics on non-parititioned tables */

	   IF b_analyze = TRUE THEN
              DBMS_Output.Put_Line('Gathering global stats:'||l_stale_tab_rec.table_name);
              log_pkg.log_info_table(l_proc_name, l_stale_tab_rec.table_name, NULL, NULL, sysdate, 'Gathering global stats : sample size '||l_global_sample_size||', degree '||l_degree||'...');
              BEGIN
                 DBMS_STATS.gather_table_stats
                   (ownname                     => l_owner_name,
                    tabname                     => l_stale_tab_rec.table_name,
                    estimate_percent            => l_global_sample_size,
                    block_sample                => b_block_sampling,
                    method_opt                  => 'FOR ALL COLUMNS SIZE AUTO',
                    degree                      => l_degree,
                    granularity                 => 'GLOBAL',
                    cascade                     => TRUE
                   );
                 log_pkg.log_info_table(l_proc_name, l_stale_tab_rec.table_name, NULL, NULL, sysdate, 'Gathered global stats');
             EXCEPTION WHEN OTHERS THEN
             IF SQLCODE != '-20000' THEN 
               log_pkg.error('PROC_ANALYZE_STALE.GLOBAL_STATS,'||l_owner_name, SQLCODE, RTRIM (SQLERRM), l_stale_tab_rec.table_name);
             ELSE
               -- Ignore UNABLE to analyze errors
               NULL;
             END IF;
           END;
         END IF;

       END IF;

   END LOOP; -- End of Gathering Stats on Monitored Tables


   /* FOR Loop to populate Global stats with statistics from a partitioned level in the event
    * that only statistics on a partitioned level were gathered from a table. This will ensure
    * that the Global stats will be uptodate and cal be used by the optimiser.
    */
   BEGIN
    FOR l_list_stale_rec IN l_list_stale_cur (l_owner_name)
    LOOP
        FOR l_get_stats_rec IN l_get_stats_cur (l_list_stale_rec.table_name)
        LOOP
           BEGIN
               DBMS_Output.Put_Line('Set global Stats:'||l_get_stats_rec.table_name||','||l_get_stats_rec.sum_rows);
               log_pkg.log_info_table(l_proc_name, l_get_stats_rec.table_name, NULL, NULL, sysdate, 'Set global Stats:'||l_get_stats_rec.table_name||', rows='||l_get_stats_rec.sum_rows);
               DBMS_STATS.set_table_stats
                   (ownname            =>  l_owner_name,
                    tabname            =>  l_get_stats_rec.table_name,
                    numrows            =>  l_get_stats_rec.sum_rows,
                    numblks            =>  l_get_stats_rec.sum_blocks,
                    avgrlen            =>  l_get_stats_rec.avg_rlen
                   );

               log_pkg.log_info_table(l_proc_name, l_get_stats_rec.table_name, NULL, NULL, sysdate, 'Global Stats updated');
               EXCEPTION WHEN OTHERS THEN
                   log_pkg.error('PROC_ANALYZE_STALE.SET_STATS,'||l_owner_name, SQLCODE, RTRIM (SQLERRM), l_get_stats_rec.table_name);

               END;
        END LOOP;
    END LOOP;
  END;  -- End Setting of Global Stats Section.
    log_pkg.log_info_table(l_proc_name, NULL, NULL, NULL, sysdate, 'Finished');

END; -- End PROC_ANALYSE_STALE Procedure
/
SHOW ERROR
