#!/bin/ksh

INSTALL_LOC=$(dirname $0)/../
LIBDIR=${INSTALL_LOC}/lib
LIBFILE=${LIBDIR}/flcc_functions.kshlib
FLCC_ROOT=${HOME}/flcc
LOGDIR=${INSTALL_LOC}/logs
LOGFILE=${LOGDIR}/flcc_installation.$$.log
rm -f ${LOGDIR}/flcc_errchk.*
ERRCHKFILE=${LOGDIR}/flcc_errchk.$$
mkdir -p ${LOGDIR}
touch ${LOGFILE}

INSTANCE_FILES=${INSTALL_LOC}/db/instance
LOADER_FILES=${INSTALL_LOC}/db/loader
SYSADM_FILES=${INSTALL_LOC}/db/schema/admin
SCHEMA_FILES=${INSTALL_LOC}/db/schema/flcc

ORAHOME=$ORACLE_HOME
ORASID=ccdb
ORASYSPWD=Sysdb01
ORASYSTEMPWD=Sysdb01
ORAUSER=etlserver
ETLUSER=${ORAUSER}
ORAPWD=Etlserver01
DB_TYPE=flcc_small
HOSTNAME=$(hostname)
DB_HOST=${HOSTNAME}
ORACLE_PORT=1521

. ${LIBFILE}
if [ $? -ne 0 ]
then
  echo "ERROR - unable to source ${LIBFILE}."
  exit 1
fi

if [[ $# -eq 0 ]];then           
    echo "Usage: $0 [ -create_database ] [ -install_schema ]"
    exit 1
fi

while [ $# -gt 0 ]
do
  case $1 in

    -create_database)
           CREATE_DATABASE=YES
           shift
           ;;

    -install_schema)
           INSTALL_SCHEMA=YES
           shift
           ;;
         *)
           echo "Usage: $0 [ -create_database ] [ -install_schema ]"
           exit 1
           ;;

   esac
done

create_database() {

   check_db_running $ORASID
   
   if [ $? -eq 0 ]
   then
     echo "WARNING: ${ORASID} Database is already running on this machine." 
     # exit 1
     return 1
   fi
   
   
   print -n "Creating database ${ORASID}..."
   dbca -silent -createDatabase \
				-templateName ${INSTANCE_FILES}/${DB_TYPE}.dbt \
				-sid "${ORASID}" \
				-gdbName "${ORASID}" \
				-sysPassword "${ORASYSPWD}" \
				-systemPassword "${ORASYSTEMPWD}" \
				-emConfiguration NONE | tee -a ${LOGFILE} 2>&1
   if [ $? -eq 0 ]
   then
     print "done"
   else
     print "failed."
     print "... exiting"
     print ""
     exit 1
   fi

   if [ -f /var/opt/oracle/oratab ]
   then
     ORATAB=/var/opt/oracle/oratab
   else
     ORATAB=/etc/oratab
   fi
   sed -e "/^${ORASID}/ s/:N/:Y/" ${ORATAB} >>/tmp/oratab.tmp
   [ $? -ne 0 ] && echo "WARNING: Failed to update ${ORATAB}" >/dev/null
   mv /tmp/oratab.tmp ${ORATAB} 2>/dev/null; chown oracle ${ORATAB} 2>/dev/null

   return 0

} # create_database

configure_netca() {

  echo "Configuring Oracle NET for SID ${ORASID}..."

  NETCA_RESP=/tmp/netca_resp.$$

  cat >${NETCA_RESP} <<EOF
[GENERAL]
RESPONSEFILE_VERSION="10.0"
CREATE_TYPE="CUSTOM"
SHOW_GUI=false
LOG_FILE=""/tmp/netca.log""
[oracle.net.ca]
INSTALLED_COMPONENTS={"server","net8"}
INSTALL_TYPE=""custom""
LISTENER_NUMBER=1
LISTENER_NAMES={"LISTENER"}
LISTENER_PROTOCOLS={"TCP;${ORACLE_PORT}"}
LISTENER_START=""LISTENER""
NAMING_METHODS={"TNSNAMES"}
NSN_NUMBER=1
NSN_NAMES={"${ORASID}"}
NSN_SERVICE={"${ORASID}"}
NSN_PROTOCOLS={"TCP;${HOSTNAME};${ORACLE_PORT}"}
EOF

  ${ORACLE_HOME}/bin/netca /silent /responsefile ${NETCA_RESP} >/dev/null
  if [ $? = 0 ]
  then
    echo "Oracle NET configuration complete for SID ${ORASID}."
    rm -f ${NETCA_RESP} /tmp/netca.log
    return 0
   else
     MSG_TO_IGNORE="LISTENER:The information provided for this listener is currently in use by another listener on this computer."
     grep "${MSG_TO_IGNORE}" /tmp/netca.log >/dev/null
     if [ $? -eq 0 ]
     then
       echo "Oracle NET configuration complete for SID ${ORASID}."
       rm -f ${NETCA_RESP} /tmp/netca.log
       return 0
     else
       echo "Oracle NET configuration failed for SID ${ORASID}."
       echo "See /tmp/netca.log for details."
       return 1
     fi
  fi

} # configure_netca

install_instance()
{

	 print -n "Installing Instance..."
     sed -e s%"\[\[ETLUSER_PASSWD\]\]"%"${ETLPASSWORD}"% ${INSTANCE_FILES}/create_user.sql >/tmp/create_user.sql.$$
     mv /tmp/create_user.sql.$$ ${INSTANCE_FILES}/create_user.sql
     cat ${INSTANCE_FILES}/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${INSTANCE_FILES}/"% | \
         ${ORAHOME}/bin/sqlplus "${SYS_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE 
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
}

install_admin_packages()
{
	 print -n "Installing Types..."
     cat ${SYSADM_FILES}/types/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/types/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
     
	 print -n "Installing Tables..."
     cat ${SYSADM_FILES}/tables/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/tables/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"

     
	 print -n "Installing Sequences..."
     cat ${SYSADM_FILES}/sequences/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/sequences/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
     
	 print -n "Installing Functions..."
     cat ${SYSADM_FILES}/functions/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/functions/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE > /dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
     
	 print -n "Installing Views..."
     cat ${SYSADM_FILES}/views/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/views/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"   
     
	 print -n "Installing Triggers..."
     cat ${SYSADM_FILES}/triggers/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/triggers/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"

	 print -n "Installing Synonyms..."
     cat ${SYSADM_FILES}/synonym/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/synonym/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE > /dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"


	 print -n "Installing Packages..."
     cat ${SYSADM_FILES}/packages/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/packages/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
    
	 print -n "Installing Procedures..."
     cat ${SYSADM_FILES}/procedures/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/procedures/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
}

install_flcc_schema()
{
	 print -n "Installing Core Tables..."
     cat ${SCHEMA_FILES}/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SCHEMA_FILES}/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
}

install_partitions()
{
	 print -n "Installing Table Partitions..."
     cat ${SYSADM_FILES}/misc/schema.sql | \
     sed -e s%"\[\[INSTALL_LOC\]\]"%"${SYSADM_FILES}/misc/"% | \
         ${ORAHOME}/bin/sqlplus "${ETL_CONNECT_STRING}" >$ERRCHKFILE
     if ( grep "^ORA-" $ERRCHKFILE >/dev/null ) ; then
        echo "failed"
        cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
        return 1
     fi
     cat $ERRCHKFILE >>$LOGFILE; rm -f $ERRCHKFILE
     echo "done"
}

deploy_loaders() {

  if [ ! -d ${FLCC_ROOT} ]
  then
    mkdir ${FLCC_ROOT}
  fi

  mkdir -p ${FLCC_ROOT}/logs/Voice
  mkdir -p ${FLCC_ROOT}/var/adapter/Voice/input
  mkdir -p ${FLCC_ROOT}/var/adapter/Voice/output/normal
  mkdir -p ${FLCC_ROOT}/var/adapter/Voice/output/backfill
  mkdir -p ${FLCC_ROOT}/logs/SMS
  mkdir -p ${FLCC_ROOT}/var/adapter/SMS/input
  mkdir -p ${FLCC_ROOT}/var/adapter/SMS/output/normal
  mkdir -p ${FLCC_ROOT}/var/adapter/SMS/output/backfill
  mkdir -p ${FLCC_ROOT}/logs/MMS
  mkdir -p ${FLCC_ROOT}/var/adapter/MMS/input
  mkdir -p ${FLCC_ROOT}/var/adapter/MMS/output/normal
  mkdir -p ${FLCC_ROOT}/var/adapter/MMS/output/backfill
  mkdir -p ${FLCC_ROOT}/logs/Data
  mkdir -p ${FLCC_ROOT}/var/adapter/Data/input
  mkdir -p ${FLCC_ROOT}/var/adapter/Data/output/normal
  mkdir -p ${FLCC_ROOT}/var/adapter/Data/output/backfill
  mkdir -p ${FLCC_ROOT}/bin/Data
  mkdir -p ${FLCC_ROOT}/bin/Voice
  mkdir -p ${FLCC_ROOT}/bin/SMS
  mkdir -p ${FLCC_ROOT}/bin/MMS
  mkdir -p ${FLCC_ROOT}/conf/Data/control
  mkdir -p ${FLCC_ROOT}/conf/Voice/control
  mkdir -p ${FLCC_ROOT}/conf/SMS/control
  mkdir -p ${FLCC_ROOT}/conf/MMS/control
  mkdir -p ${FLCC_ROOT}/conf/Data/xml
  mkdir -p ${FLCC_ROOT}/conf/Voice/xml
  mkdir -p ${FLCC_ROOT}/conf/SMS/xml
  mkdir -p ${FLCC_ROOT}/conf/MMS/xml
  mkdir -p ${FLCC_ROOT}/conf/LAST_NWK_TSTAMP
  mkdir -p ${FLCC_ROOT}/logs/LAST_NWK_TSTAMP

  cp ${LOADER_FILES}/Data/bin/* ${FLCC_ROOT}/bin/Data
  cp ${LOADER_FILES}/Voice/bin/* ${FLCC_ROOT}/bin/Voice
  cp ${LOADER_FILES}/SMS/bin/* ${FLCC_ROOT}/bin/SMS
  cp ${LOADER_FILES}/MMS/bin/* ${FLCC_ROOT}/bin/MMS

  cp ${LOADER_FILES}/Data/control/* ${FLCC_ROOT}/conf/Data/control
  cp ${LOADER_FILES}/Voice/control/* ${FLCC_ROOT}/conf/Voice/control
  cp ${LOADER_FILES}/SMS/control/* ${FLCC_ROOT}/conf/SMS/control
  cp ${LOADER_FILES}/MMS/control/* ${FLCC_ROOT}/conf/MMS/control

  cp ${LOADER_FILES}/Data/xml/* ${FLCC_ROOT}/conf/Data/xml
  cp ${LOADER_FILES}/Voice/xml/* ${FLCC_ROOT}/conf/Voice/xml
  cp ${LOADER_FILES}/SMS/xml/* ${FLCC_ROOT}/conf/SMS/xml
  cp ${LOADER_FILES}/MMS/xml/* ${FLCC_ROOT}/conf/MMS/xml

  cat ${LOADER_FILES}/common.properties  | \
  sed -e s%"\[\[ORACLE_HOME\]\]"%"${ORACLE_HOME}/"% | \
  sed -e s%"\[\[FLCC_ROOT\]\]"%"${FLCC_ROOT}/"% > ${FLCC_ROOT}/bin/common.properties

} # deploy_loaders

gather_env_details() {

  while :
  do
    print -n "Enter FLCC Home (${FLCC_ROOT}): "
    read SOMEVAR
    [ -z "${SOMEVAR}" ] || FLCC_ROOT=${SOMEVAR}
    if [ "$FLCC_ROOT" == "" ]
    then
      echo "Empty Value Entered"
      continue
    else
      break
    fi
  done
  
  while :
  do
    print -n "Enter Oracle Home (${ORAHOME}): "
    read SOMEVAR
    [ -z "${SOMEVAR}" ] || ORAHOME=${SOMEVAR}
    if [ ! -f $ORAHOME/bin/sqlplus ]
    then
      echo "Invalid path specified."
      continue
    else
      break
    fi
  done
  export ORACLE_HOME=$ORAHOME
    
  while :
  do    
    print -n "Enter Oracle SID (${ORASID}): "    	
    read SOMEVAR
    [ -z "${SOMEVAR}" ] || ORASID=${SOMEVAR}
    if [ "$ORASID" == "" ]
    then
      echo "Empty Value Entered"
      continue
    else
      break
    fi
  done
  export ORACLE_SID=$ORASID

  while :
  do    
    print -n "Enter Oracle password for user 'sys': "    
    stty -echo
    read SOMEVAR
    stty echo	
    [ -z "${SOMEVAR}" ] || ORASYSPWD=${SOMEVAR}
    if [ "$ORASYSPWD" == "" ]
    then
      echo "Empty Value Entered"
      continue
    else
      break
    fi
  done
  echo

  while :
  do    
    print -n "Enter Oracle password for user 'system': "    
    stty -echo
    read SOMEVAR
    stty echo	
    [ -z "${SOMEVAR}" ] || ORASYSTEMPWD=${SOMEVAR}
    if [ "$ORASYSTEMPWD" == "" ]
    then
      echo "Empty Value Entered"
      continue
    else
      break
    fi
  done
  echo
	
  while :
  do    
    print -n "Enter Oracle password for user '${ORAUSER}': "    
    stty -echo
    read SOMEVAR
    stty echo	
    [ -z "${SOMEVAR}" ] || ORAPWD=${SOMEVAR}
    if [ "$ORAPWD" == "" ]
    then
      echo "Empty Value Entered"
      continue
    else
      break
    fi
  done
  echo
  export ETLPASSWORD=$ORAPWD

  ETL_CONNECT_STRING="${ORAUSER}/${ORAPWD}@${ORASID}"
  SYS_CONNECT_STRING="sys/${ORASYSPWD}@${ORASID} as sysdba"
    
  while :
  do    
    print -n "Enter the database template name (${DB_TYPE}): "    	
    read SOMEVAR
    [ -z "${SOMEVAR}" ] || DB_TYPE=${SOMEVAR}
    if [ "$DB_TYPE" == "" ]
    then
      echo "Empty Value Entered"
      continue
    else
      break
    fi
  done

} # gather_env_details

# main|Main|MAIN

if [ "${CREATE_DATABASE}" = "YES" ]
then
  gather_env_details
  create_database
  configure_netca
  start_listeners
  wait_for_oracle_listener ${ORASID}
elif  [ "${INSTALL_SCHEMA}" = "YES" ]
then
  gather_env_details
  install_instance 
  install_admin_packages
  install_flcc_schema
  install_partitions
fi

echo "See log file ${LOGFILE} for more details."

